# tranchitella.recipe.wsgi
# Copyright (C) 2008-2010 Tranchitella Kft. <http://tranchtella.eu>

import logging
import os
import string

from zc.buildout.easy_install import scripts
from zc.recipe.egg.egg import Eggs


WRAPPER_TEMPLATE = """\
import sys
sys.path[0:0] = [
    %(syspath)s,
]
%(environ)s
from paste.deploy import loadapp
application = loadapp("config:%(config)s")
"""


class Recipe(object):
    """Buildout recipe: tranchitella.recipe.wsgi:default"""

    def __init__(self, buildout, name, options):
        self.buildout = buildout
        self.name = name
        self.options = options
        self.options['eggs'] += '\nPasteDeploy'
        self.script_name = options.get('script-name', self.name)

    def install(self):
        egg = Eggs(self.buildout, self.options["recipe"], self.options)
        requirements, ws = egg.working_set()
        path = [pkg.location for pkg in ws]
        extra_paths = self.options.get('extra-paths', '')
        extra_paths = extra_paths.split()
        path.extend(extra_paths)
        environ = self.options.get('environ', '')
        if environ:
            environ = filter(None, environ.splitlines())
            environ = '\nimport os\n' + ''.join([
                "os.environ['%s'] = '%s'\n" % \
                    tuple(map(string.strip, i.split('=', 1))) \
                for i in environ if '='  in i])
        output = WRAPPER_TEMPLATE % dict(
            config=self.options["config-file"], environ=environ,
            syspath=",\n    ".join((repr(p) for p in path)))
        location = self.buildout["buildout"]["bin-directory"]
        target = os.path.join(location, self.script_name)
        open(target, "wt").write(output)
        os.chmod(target, 0755)
        self.options.created(target)
        return self.options.created()

    def update(self):
        self.install()



# tranchitella.recipe.wsgi
# Copyright (C) 2008-2010 Tranchitella Kft. <http://tranchtella.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from setuptools import setup, find_packages

install_requires = [
    'setuptools',
    'zc.buildout',
    'zc.recipe.egg',
]

tests_require = []

extras_require = dict(
    test=tests_require,
)

setup(
    name='tranchitella.recipe.wsgi',
    version='0.4',
    url='http://pypi.python.org/pypi/tranchitella.recipe.wsgi',
    license='GPL 2',
    author='Tranchitella Kft.',
    author_email='info@tranchitella.eu',
    description="Buildout recipe to create paste.deploy entry points for mod_wsgi",
    long_description=(
        open('README.txt').read() + '\n\n' +
        open('CHANGES.txt').read()
    ),
    classifiers=[
        "Framework :: Buildout",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
    ],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['tranchitella', 'tranchitella.recipe'],
    install_requires=install_requires,
    extras_require=extras_require,
    tests_require=install_requires + tests_require,
    test_suite="tranchitella.recipe.wsgi.tests",
    include_package_data=True,
    zip_safe=False,
    entry_points="""
    [zc.buildout]
    default = tranchitella.recipe.wsgi:Recipe
    """
)



# tranchitella.recipe.wsgi
# Copyright (C) 2008-2010 Tranchitella Kft. <http://tranchtella.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import doctest
import unittest

from zc.buildout import testing


def buildoutSetUp(test):
    testing.buildoutSetUp(test)
    testing.install_develop('tranchitella.recipe.wsgi', test)


class RecipeUnitTest(unittest.TestCase):

    def __new__(self, test):
        return getattr(self, test)()

    @classmethod
    def test_recipe(cls):
        return doctest.DocFileSuite('recipe.txt',
            setUp=buildoutSetUp, tearDown=testing.buildoutTearDown,
            optionflags=(doctest.ELLIPSIS | doctest.REPORT_ONLY_FIRST_FAILURE))
